@@ -1,16 +1,14 @@
 use std::{fs::File, io, io::Read, path::Path};
 
-use classfmt::parser::ClassParser;
+use classfmt::ClassParser;
 
 fn main() -> io::Result<()> {
     let mut f = File::open(Path::new("./tests/Fields.class"))?;
     let mut buf = Vec::with_capacity(64);
-
     f.read_to_end(&mut buf)?;
 
-    let mut parser = ClassParser::from_bytes(&buf);
-    let class = parser.parse().unwrap();
-    println!("{:#?}", class);
+    let class = ClassParser::from_bytes(&buf).parse().unwrap();
 
+    println!("{:#?}", class);
     Ok(())
 }
