@@ -5,11 +5,11 @@ pub mod access_flags;
 pub mod attribute;
 pub mod constant_pool;
 pub mod error;
-pub mod field;
-pub mod method;
+mod field;
+mod method;
 pub mod opcode;
-pub mod parser;
-pub mod raw_class;
+mod parser;
+mod raw_class;
 
 pub use attribute::Attribute;
 pub use constant_pool::{Constant, ConstantTag, ReferenceKind};
